import 'package:flutter/material.dart';
import 'package:tim_ui_kit_sticker_plugin/utils/tim_ui_kit_sticker_data.dart';

class StickerPanel extends StatefulWidget {
  final void Function() sendTextMsg;
  final void Function(int index, String data) sendFaceMsg;
  final void Function(int unicode) addText;
  final void Function() deleteText;
  final List<CustomStickerPackage> customStickerPackageList;
  final bool showBottomContainer;
  final Color? backgroundColor;
  final Color? lightPrimaryColor;

  const StickerPanel({
    Key? key,
    required this.sendTextMsg,
    required this.sendFaceMsg,
    required this.deleteText,
    required this.addText,
    required this.customStickerPackageList,
    this.backgroundColor = const Color(0xFFEDEDED),
    this.lightPrimaryColor = const Color(0xFF3371CD),
    this.showBottomContainer = true, // 可选参数，是否展示下方的底部导航栏
  }) : super(key: key);
  @override
  State<StatefulWidget> createState() => _EmojiPanelState();
}

class _EmojiPanelState extends State<StickerPanel> {
  int selectedIdx = 0;
  late List<int> textEmojiIndexList;

  @override
  void initState() {
    super.initState();
    filterTextEmojiIndexList();
  }

  void filterTextEmojiIndexList() {
    textEmojiIndexList = [];
    widget.customStickerPackageList
        .asMap()
        .keys
        .forEach((customStickerPackageIndex) {
      if (!widget.customStickerPackageList[customStickerPackageIndex]
          .isCustomSticker) {
        textEmojiIndexList.add(customStickerPackageIndex);
      }
    });
  }

  @override
  didChangeDependencies() {
    super.didChangeDependencies();
    filterTextEmojiIndexList();
  }

  List<Widget> _buildEmojiListWidget(
      List<CustomStickerPackage> customStickerList) {
    List<Widget> list = [];
    for (var index = 0; index < (customStickerList.length); index++) {
      final customEmojiFace = customStickerList[index];

      list.add(Container(
          padding: const EdgeInsets.symmetric(horizontal: 8, vertical: 2),
          decoration: BoxDecoration(
              color: selectedIdx == index
                  ? widget.lightPrimaryColor
                  : widget.backgroundColor,
              borderRadius: const BorderRadius.all(Radius.circular(4))),
          child: InkWell(
              onTap: () {
                setState(() {
                  selectedIdx = index;
                });
              },
              child: customEmojiFace.isCustomSticker
                  ? CustomEmojiItem(
                      sticker: customEmojiFace.menuItem,
                      baseUrl: customEmojiFace.baseUrl)
                  : EmojiItem(
                      name: customEmojiFace.menuItem.name,
                      unicode: customEmojiFace.menuItem.unicode!,
                    ))));
    }
    return list;
  }

  Widget _buildEmojiPanel(List<int> textEmojiIndexList,
      List<CustomStickerPackage> customStickerList) {
    assert(widget.customStickerPackageList.isNotEmpty,
        'customStickerPackageList can not be empty');
    if (textEmojiIndexList.contains(selectedIdx)) {
      return Stack(
        children: [
          GridView(
            gridDelegate: const SliverGridDelegateWithFixedCrossAxisCount(
              crossAxisCount: 8,
              childAspectRatio: 1,
            ),
            children: customStickerList[selectedIdx].stickerList.map(
              (item) {
                return InkWell(
                  onTap: () {
                    widget.addText(item.unicode!);
                  },
                  child: EmojiItem(
                    name: item.name,
                    unicode: item.unicode!,
                  ),
                );
              },
            ).toList(),
          ),
          Align(
            alignment: Alignment.bottomRight,
            child: SingleChildScrollView(
              child: GestureDetector(
                onTap: () {
                  widget.deleteText();
                },
                child: Container(
                    color: widget.backgroundColor,
                    margin: const EdgeInsets.only(right: 15),
                    width: 35,
                    child: Center(
                      child: Image.asset(
                        'images/delete_emoji.png',
                        package: 'tim_ui_kit_sticker_plugin',
                        width: 35,
                        height: 20,
                      ),
                    )),
              ),
            ),
          ),
        ],
      );
    }
    return GridView(
      gridDelegate: const SliverGridDelegateWithFixedCrossAxisCount(
        crossAxisCount: 4,
        childAspectRatio: 1,
      ),
      children: customStickerList[selectedIdx].stickerList.map(
        (item) {
          return InkWell(
            onTap: () {
              widget.sendFaceMsg(
                  item.index,
                  customStickerList[selectedIdx].baseUrl == null
                      ? item.url!
                      : customStickerList[selectedIdx].baseUrl! +
                          '/' +
                          item.name);
            },
            child: CustomEmojiItem(
                isBigImage: true,
                baseUrl: customStickerList[selectedIdx].baseUrl,
                sticker: item),
          );
        },
      ).toList(),
    );
  }

  Widget _buildBottomPanel(List<int> textEmojiIndexList,
      List<CustomStickerPackage> customStickerList) {
    assert(widget.customStickerPackageList.isNotEmpty,
        'customStickerPackageList can not be empty');
    return SizedBox(
        height: 40,
        child: Row(children: [
          Expanded(
              child: Container(
                  margin: const EdgeInsets.only(right: 25),
                  child: SingleChildScrollView(
                    scrollDirection: Axis.horizontal,
                    child:
                        Row(children: _buildEmojiListWidget(customStickerList)),
                  ))),
          if (textEmojiIndexList.contains(selectedIdx))
            ElevatedButton(
                child: const Text("发送"),
                style: ElevatedButton.styleFrom(),
                onPressed: () {
                  widget.sendTextMsg();
                })
        ]));
  }

  @override
  Widget build(BuildContext context) {
    return GestureDetector(
        onTap: () {},
        child: SingleChildScrollView(
            child: Column(
          children: [
            Container(
              height: widget.showBottomContainer ? 190 : 248,
              color: widget.backgroundColor,
              padding: const EdgeInsets.fromLTRB(24, 16, 24, 16),
              child: _buildEmojiPanel(
                  textEmojiIndexList, widget.customStickerPackageList),
            ),
            widget.showBottomContainer
                ? Container(
                    margin: const EdgeInsets.fromLTRB(16, 0, 32, 0),
                    child: _buildBottomPanel(
                        textEmojiIndexList, widget.customStickerPackageList))
                : Container()
          ],
        )));
  }
}

class EmojiItem extends StatelessWidget {
  const EmojiItem({Key? key, required this.name, required this.unicode})
      : super(key: key);
  final String name;
  final int unicode;
  @override
  Widget build(BuildContext context) {
    return Text(
      String.fromCharCode(unicode),
      style: const TextStyle(
        fontSize: 26,
      ),
    );
  }
}

class CustomEmojiItem extends StatelessWidget {
  const CustomEmojiItem(
      {Key? key, required this.sticker, this.baseUrl, this.isBigImage = false})
      : super(key: key);
  final CustomSticker sticker;
  final String? baseUrl;
  final bool? isBigImage;

  bool isFromNetwork() {
    if (baseUrl == null) {
      return sticker.url!.startsWith('http');
    }
    return baseUrl!.startsWith('http');
  }

  String getUrl() {
    return baseUrl == null ? sticker.url! : baseUrl! + '/' + sticker.name;
  }

  double get size => isBigImage! ? 60 : 30;

  @override
  Widget build(BuildContext context) {
    return Container(
        child: isFromNetwork()
            ? Image.network(
                getUrl(),
                height: size,
                width: size,
              )
            : Image.asset(
                getUrl(),
                height: size,
                width: size,
              ));
  }
}
